@@ -36,6 +36,8 @@
 #include <libgnomecups/gnome-cups-ui-print.h>
 #include <libgnomecups/gnome-cups-ui-util.h>
 #include <libgnomecups/gnome-cups-ui-driver.h>
+#include <libgnomecups/gnome-cups-ui-connection.h>
+#include <libgnomecups/gnome-cups-permission.h>
 
 #include <cups/cups.h>
 
@@ -44,12 +46,6 @@ typedef struct {
 	char *value;
 } PrinterInfoData;
 
-static gboolean
-need_admin_priviledges (void)
-{
-	return (geteuid () != 0);
-}
-
 static void
 unref_group (gpointer user_data, GObject *obj)
 {
@@ -62,56 +58,61 @@ setup_size_group (GladeXML *xml,
 		  int n_items,
 		  const char **items)
 {
-	GtkSizeGroup *group;
 	int i;
+	GtkSizeGroup *group = gtk_size_group_new (mode);
 
-	group = gtk_size_group_new (mode);
-
-	for (i = 0; i < n_items; i++) {
-		GtkWidget *widget;
-
-		widget = glade_xml_get_widget (xml, items[i]);
-		gtk_size_group_add_widget (group, widget);
-	}
-
+	for (i = 0; i < n_items; i++)
+		gtk_size_group_add_widget (group,
+			glade_xml_get_widget (xml, items[i]));
 	g_object_weak_ref (G_OBJECT (xml), unref_group, group);
 }
 
 static void
-disconnect_view (gpointer user_data, GObject *object)
+disconnect_view (gpointer printer, GObject *object)
 {
-	GnomeCupsPrinter *printer = user_data;
 	g_object_set_data (G_OBJECT (printer), "properties-window", NULL);
 }
 
+static void
+label_vs_entry (GladeXML *xml,
+		char const *entry, char const *label, char const *val)
+{
+	GtkWidget *w = glade_xml_get_widget (xml, entry);
+	if (gnome_cups_can_admin ())
+		gtk_entry_set_text (GTK_ENTRY (w), val);
+	else
+		gtk_widget_hide (w);
+	w = glade_xml_get_widget (xml, label);
+	if (gnome_cups_can_admin ())
+		gtk_widget_hide (w);
+	else
+		gtk_label_set_text (GTK_LABEL (w), val);
+}
+
 static void
 update_general_page (GladeXML *xml)
 {
-	GnomeCupsPrinter *printer;
-	GtkWidget *widget;
+	GnomeCupsPrinter *printer = g_object_get_data (G_OBJECT (xml), "printer");
+	GtkWidget *w;
 	GdkPixbuf *pixbuf;
 	char	  *icon_name;
 
-	printer = g_object_get_data (G_OBJECT (xml), "printer");
-	widget = glade_xml_get_widget (xml, "printer_image");
 	gnome_cups_printer_get_icon (printer, &icon_name, NULL);
-
 	pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
 		icon_name, 48, GTK_ICON_LOOKUP_USE_BUILTIN, NULL);
-	gtk_image_set_from_pixbuf (GTK_IMAGE (widget), pixbuf);
+	w = glade_xml_get_widget (xml, "printer_image");
+	gtk_image_set_from_pixbuf (GTK_IMAGE (w), pixbuf);
 
-	widget = glade_xml_get_widget (xml, "name_entry");
-	gtk_entry_set_text (GTK_ENTRY (widget),
-			    gnome_cups_printer_get_name (printer));
-	widget = glade_xml_get_widget (xml, "description_entry");
-	gtk_entry_set_text (GTK_ENTRY (widget),
-			    gnome_cups_printer_get_description (printer));
-	widget = glade_xml_get_widget (xml, "location_entry");
-	gtk_entry_set_text (GTK_ENTRY (widget),
-			    gnome_cups_printer_get_location (printer));
-	widget = glade_xml_get_widget (xml, "status_label");
-	gtk_label_set_text (GTK_LABEL (widget),
-			    gnome_cups_printer_get_full_state (printer));
+	label_vs_entry (xml, "name_entry", "name_label",
+		gnome_cups_printer_get_name (printer));
+	label_vs_entry (xml, "description_entry", "description_label",
+		gnome_cups_printer_get_description (printer));
+	label_vs_entry (xml, "location_entry", "location_label",
+		gnome_cups_printer_get_location (printer));
+
+	w = glade_xml_get_widget (xml, "status_label");
+	gtk_label_set_text (GTK_LABEL (w),
+		gnome_cups_printer_get_full_state (printer));
 }
 
 static gboolean
@@ -389,14 +390,17 @@ setup_general_page (GladeXML *xml)
 
 	update_general_page (xml);
 
-	widget = glade_xml_get_widget (xml, "description_entry");
-	if (need_admin_priviledges ())
-	g_signal_connect (widget, "focus_out_event",
-			  G_CALLBACK (description_focus_out_cb), xml);
+	if (gnome_cups_can_admin ()) {
+		widget = glade_xml_get_widget (xml, "description_entry");
+		g_signal_connect (widget,
+			"focus_out_event",
+			G_CALLBACK (description_focus_out_cb), xml);
 
-	widget = glade_xml_get_widget (xml, "location_entry");
-	g_signal_connect (widget, "focus_out_event",
-			  G_CALLBACK (location_focus_out_cb), xml);
+		widget = glade_xml_get_widget (xml, "location_entry");
+		g_signal_connect (widget,
+			"focus_out_event",
+			G_CALLBACK (location_focus_out_cb), xml);
+	}
 
 	hookup_option_menu_xml (xml, "resolution_option_menu", "Resolution");
 }
@@ -579,7 +583,7 @@ cb_driver_changed (GCupsDriverSelector *ds, GnomeCupsPrinter *printer)
 
 	if (ppd != NULL) {
 		ipp_t *request = gnome_cups_request_new_for_printer (
-			CUPS_ADD_PRINTER, gnome_cups_printer_get_name (printer));
+			CUPS_ADD_PRINTER, printer);
 		ippAddString (request, IPP_TAG_PRINTER, IPP_TAG_NAME,
 			"ppd-name", NULL, gnome_cups_strdup (ppd->filename));
 		gnome_cups_request_execute_async (request, NULL, "/admin/", NULL, NULL, NULL);
@@ -593,7 +597,7 @@ setup_driver_page (GladeXML *xml, GnomeCupsPrinter *printer)
 	ppd_file_t *ppd;
 	ppd_attr_t *nickname;
 
-	if (need_admin_priviledges ())
+	if (!gnome_cups_can_admin ())
 		return;
 	if (printer == NULL)
 		return;
@@ -616,32 +620,48 @@ setup_driver_page (GladeXML *xml, GnomeCupsPrinter *printer)
 			G_CALLBACK (cb_driver_changed), xml, 0);
 	}
 	ppdClose (ppd);
-
 }
 
-#define SU_APP	"gnomesu"
+static void
+cb_connection_changed (GCupsConnectionSelector *cs, GnomeCupsPrinter *printer)
+{
+}
+static void
+setup_connection_page (GladeXML *xml, GnomeCupsPrinter *printer)
+{
+	GtkWidget  *w, *cs;
+	char const *uri;
+
+	if (!gnome_cups_can_admin ())
+		return;
+	if (printer == NULL)
+		return;
+	uri = gnome_cups_printer_get_device_uri (printer);
+	if (uri == NULL)
+		return;
+
+	cs = gcups_connection_selector_new ();
+	w = glade_xml_get_widget (xml, "prop_notebook");
+	gtk_notebook_append_page (GTK_NOTEBOOK (w),
+		cs, gtk_label_new (_("Connection")));
+	gcups_connection_selector_set_uri (GCUPS_CONNECTION_SELECTOR (cs),
+		uri);
+	gtk_widget_show (cs);
+
+	g_signal_connect_object (cs,
+		"changed",
+		G_CALLBACK (cb_connection_changed), xml, 0);
+}
+
 static void
 cb_become_admin (GnomeCupsPrinter *printer)
 {
-	GError *err = NULL;
-	char *argv[] = { NULL, "gnome-cups-manager", "-p", NULL, NULL };
-	argv[0] = (char *)SU_APP;
-	argv[2] = (char *)gnome_cups_printer_get_name (printer);
-	g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
-		NULL, NULL, NULL, &err);
-
-	if (err != NULL) {
-		GtkWidget *dialog = gtk_message_dialog_new_with_markup (NULL, 
-			 GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL, 
-			 GTK_MESSAGE_ERROR,
-			 GTK_BUTTONS_OK,
-			_("Problems launching %s as root via %s\n%s"),
-			argv[1], SU_APP, err->message);
-		gtk_dialog_run (GTK_DIALOG (dialog));
-		gtk_widget_destroy (dialog); 
-		g_error_free (err);
-	}
-	gtk_main_quit ();
+	char const *argv[] = { "-p", NULL };
+	GladeXML *xml = g_object_get_data (G_OBJECT (printer), "properties-window");
+	argv[1] = gnome_cups_printer_get_name (printer);
+	if (gnome_cups_spawn ("gnome-cups-manager", G_N_ELEMENTS (argv), argv,
+			      TRUE, glade_xml_get_widget (xml, "properties_window")))
+		gtk_main_quit ();
 }
 
 void
@@ -689,14 +709,15 @@ gnome_cups_manager_printer_properties (GnomeCupsPrinter *printer)
 	setup_paper_page (xml);
 	setup_advanced_page (xml);
 	setup_driver_page (xml, printer);
+	setup_connection_page (xml, printer);
 
 	button = glade_xml_get_widget (xml, "admin_button");
-	if (need_admin_priviledges ())
+	if (gnome_cups_can_admin ())
+		gtk_widget_hide (button);
+	else
 		g_signal_connect_swapped (button,
 			"clicked",
 			G_CALLBACK (cb_become_admin), printer);
-	else
-		gtk_widget_hide (button);
 
 	button = glade_xml_get_widget (xml, "test_page_button");
 	g_signal_connect (button,
